@@ -51,7 +51,7 @@ class CompoundMixin(object):
     def contains_pts(self, pts):
         if len(self.objects) == 0:
             x_arr, y_arr = np.asarray(pts).T
-            return np.full(x_arr.shape, False, dtype=np.bool)
+            return np.full(x_arr.shape, False, dtype=bool)
 
         return reduce(self._contains_reduce,
                       map(lambda obj: obj.contains_pts(pts), self.objects))
@@ -144,7 +144,7 @@ class CompoundMixin(object):
             self.delete_object(obj)
 
     def delete_all_objects(self):
-        self.objects[:] = []
+        self.objects.clear()
 
     def roll_objects(self, n):
         num = len(self.objects)
@@ -167,6 +167,9 @@ class CompoundMixin(object):
 
     def add_object(self, obj, belowThis=None):
 
+        if obj in self.objects:
+            raise ValueError("object is already in this compound object")
+
         obj.initialize(self, self.viewer, self.logger)
 
         if belowThis is None:
